@@ -764,14 +764,14 @@ nautilus_drag_autoscroll_start (NautilusDragInfo *drag_info,
 			drag_info->start_auto_scroll_in = eel_get_system_time() 
 				+ AUTOSCROLL_INITIAL_DELAY;
 			
-			drag_info->auto_scroll_timeout_id = gtk_timeout_add
+			drag_info->auto_scroll_timeout_id = g_timeout_add
 				(AUTOSCROLL_TIMEOUT_INTERVAL,
 				 callback,
 			 	 user_data);
 		}
 	} else {
 		if (drag_info->auto_scroll_timeout_id != 0) {
-			gtk_timeout_remove (drag_info->auto_scroll_timeout_id);
+			g_source_remove (drag_info->auto_scroll_timeout_id);
 			drag_info->auto_scroll_timeout_id = 0;
 		}
 	}
@@ -781,7 +781,7 @@ void
 nautilus_drag_autoscroll_stop (NautilusDragInfo *drag_info)
 {
 	if (drag_info->auto_scroll_timeout_id != 0) {
-		gtk_timeout_remove (drag_info->auto_scroll_timeout_id);
+		g_source_remove (drag_info->auto_scroll_timeout_id);
 		drag_info->auto_scroll_timeout_id = 0;
 	}
 }
