@@ -80,7 +80,7 @@ struct NautilusFileDetails
 
 	/* Info you might get from a link (.desktop, .directory or nautilus link) */
 	char *display_name;
-	char *custom_icon_uri;
+	char *custom_icon;
 	char *activation_uri;
 
 	/* The following is for file operations in progress. Since
@@ -153,6 +153,10 @@ gboolean      nautilus_file_update_info                    (NautilusFile
 gboolean      nautilus_file_update_name                    (NautilusFile           *file,
 							    const char             *name);
 
+gboolean      nautilus_file_update_name_and_directory      (NautilusFile           *file,
+							    const char             *name,
+							    NautilusDirectory      *directory);
+
 /* Return true if the top lefts of files in this directory should be
  * fetched, according to the preference settings.
  */
@@ -168,4 +172,6 @@ gboolean      nautilus_file_is_self_owned                  (NautilusFile
 void          nautilus_file_invalidate_count_and_mime_list (NautilusFile           *file);
 gboolean      nautilus_file_rename_in_progress             (NautilusFile           *file);
 
+GnomeVFSFileInfo *nautilus_file_peek_vfs_file_info         (NautilusFile           *file);
+
 #endif
