@@ -486,7 +486,7 @@ static const PreferenceDefault preference_defaults[] = {
 	},
 	{ NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_ZOOM_LEVEL,
 	  PREFERENCE_STRING,
-	  "small",
+	  "smaller",
 	  NULL, NULL,
 	  "default_zoom_level"
 	},
