@@ -53,6 +53,10 @@ typedef struct {
 
 	/* Whether this item was selected before rubberbanding. */
 	eel_boolean_bit was_selected_before_rubberband : 1;
+
+	/* Whether a monitor was set on this icon. */
+	eel_boolean_bit is_monitored : 1;
+
 } NautilusIcon;
 
 
@@ -101,8 +105,10 @@ typedef struct {
 enum {
 	LABEL_COLOR,
 	LABEL_COLOR_HIGHLIGHT,
+	LABEL_COLOR_ACTIVE,
 	LABEL_INFO_COLOR,
 	LABEL_INFO_COLOR_HIGHLIGHT,
+	LABEL_INFO_COLOR_ACTIVE,
 	LAST_LABEL_COLOR
 };
 
@@ -155,6 +161,9 @@ struct NautilusIconContainerDetails {
 	StretchState stretch_start;
 	int context_menu_timeout_id;
 
+	gboolean icon_selected_on_button_down;
+	NautilusIcon *double_click_icon[2]; /* Both clicks in a double click need to be on the same icon */
+
 	/* Renaming Details */
 	gboolean renaming;
 	GtkWidget *rename_widget;	/* Editable text item */
@@ -183,8 +192,8 @@ struct NautilusIconContainerDetails {
 
 	/* pixbuf and color for label highlighting */
 	GdkPixbuf *highlight_frame;
-	GdkColor   highlight_color;
 	guint32    highlight_color_rgba;
+	guint32    active_color_rgba;
 	
 	/* colors for text labels */
 	GdkGC   *label_gcs    [LAST_LABEL_COLOR];
@@ -209,6 +218,9 @@ struct NautilusIconContainerDetails {
 	
 	/* Is the container fixed or resizable */
 	gboolean is_fixed_size;
+	
+	/* Is the container for a desktop window */
+	gboolean is_desktop;
 
 	/* Ignore the visible area the next time the scroll region is recomputed */
 	gboolean reset_scroll_region_trigger;
