@@ -126,7 +126,7 @@ static void
 remove_scroll_timeout (NautilusTreeViewDragDest *dest)
 {
 	if (dest->details->scroll_id) {
-		gtk_timeout_remove (dest->details->scroll_id);
+		g_source_remove (dest->details->scroll_id);
 		dest->details->scroll_id = 0;
 	}
 }
@@ -388,9 +388,9 @@ drag_motion_callback (GtkWidget *widget,
 	
 	if (dest->details->scroll_id == 0) {
 		dest->details->scroll_id = 
-			gtk_timeout_add (150, 
-					 scroll_timeout, 
-					 dest->details->tree_view);
+			g_timeout_add (150, 
+				       scroll_timeout, 
+				       dest->details->tree_view);
 	}
 
 	gdk_drag_status (context, action, time);
