@@ -25,7 +25,6 @@
 #include <config.h>
 #include "nautilus-view-identifier.h"
 
-#include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 
 
@@ -38,6 +37,7 @@ static NautilusViewIdentifier *
 nautilus_view_identifier_new (const char *iid,
 			      const char *name,
 			      const char *view_as_label,
+			      const char *view_as_label_with_mnemonic,
 			      const char *label_viewer);
 
 
@@ -46,6 +46,7 @@ NautilusViewIdentifier *
 nautilus_view_identifier_new (const char *iid, 
 			      const char *name,
 			      const char *view_as_label,
+			      const char *view_as_label_with_mnemonic,
 			      const char *viewer_label)
 {
         NautilusViewIdentifier *new_identifier;
@@ -60,6 +61,9 @@ nautilus_view_identifier_new (const char *iid,
         new_identifier->view_as_label = view_as_label ? g_strdup (view_as_label) :
 		g_strdup_printf (_("View as %s"), name);
 
+        new_identifier->view_as_label_with_mnemonic = view_as_label_with_mnemonic ? g_strdup (view_as_label_with_mnemonic) 
+		: g_strdup (new_identifier->view_as_label);
+
         new_identifier->viewer_label = view_as_label ? g_strdup (viewer_label) :
 		g_strdup_printf (_("%s Viewer"), name);
         
@@ -76,6 +80,7 @@ nautilus_view_identifier_copy (const NautilusViewIdentifier *identifier)
 	return nautilus_view_identifier_new (identifier->iid, 
 					     identifier->name, 
 					     identifier->view_as_label,
+					     identifier->view_as_label_with_mnemonic,
 					     identifier->viewer_label);
 }
 
@@ -85,69 +90,46 @@ nautilus_view_identifier_copy (const NautilusViewIdentifier *identifier)
 static GSList *
 get_lang_list (void)
 {
-        GSList *retval;
-        char *lang, *lang_with_locale, *tmp, *org_pointer;
-        char *equal_char;
+   	GSList *retval;
+	const GList *l;
+	const char *lang;
 
-        retval = NULL;
-
-        tmp = g_getenv ("LANGUAGE");
-
-        if (tmp == NULL) {
-                tmp = g_getenv ("LANG");
-        }
-
-	lang = g_strdup (tmp);
-	org_pointer = lang;
-
-	if (lang != NULL) {
-		/* envs can be in NAME=VALUE form */
-		equal_char = strchr (lang, '=');
-		if (equal_char != NULL) {
-			lang = equal_char + 1;
+	retval = NULL;
+	
+	for (l = gnome_i18n_get_language_list (NULL);
+	     l != NULL;
+	     l = g_list_next (l)) {
+		lang = l->data;
+		/* skip C locale */
+		if (l->data && strcmp (lang, "C") == 0) {
+			continue;
 		}
 		
-		/* lang may be in form LANG_LOCALE */
-		equal_char = strchr (lang, '_');
-		if (equal_char != NULL) {
-			lang_with_locale = g_strdup (lang);
-			*equal_char = 0;
-		} else {
-			lang_with_locale = NULL;
-		}
-
-		/* Make sure we don't give oaf an empty
-		   lang string */
-		if (!eel_str_is_empty (lang_with_locale)) {
-			retval = g_slist_prepend (retval, 
-						  g_strdup (lang_with_locale));
-		}
-		g_free (lang_with_locale);
 		if (!eel_str_is_empty (lang)) {
 			retval = g_slist_prepend (retval, g_strdup (lang));
 		}
-        }
-	g_free (org_pointer);
-        
+	}
         return retval;
 }
 
 NautilusViewIdentifier *
-nautilus_view_identifier_new_from_oaf_server_info (OAF_ServerInfo *server, char *name_attribute)
+nautilus_view_identifier_new_from_bonobo_server_info (Bonobo_ServerInfo *server, char *name_attribute)
 {
         const char *view_as_name;       
         const char *view_as_label;       
+	const char *view_as_label_with_mnemonic;
         const char *viewer_label;       
         GSList *langs;
 
         langs = get_lang_list ();
 
-        view_as_name = oaf_server_info_prop_lookup (server, name_attribute, langs);
-	view_as_label = oaf_server_info_prop_lookup (server, "nautilus:view_as_label", langs);
-	viewer_label = oaf_server_info_prop_lookup (server, "nautilus:viewer_label", langs);
+        view_as_name = bonobo_server_info_prop_lookup (server, name_attribute, langs);
+	view_as_label = bonobo_server_info_prop_lookup (server, "nautilus:view_as_label", langs);
+	view_as_label_with_mnemonic = bonobo_server_info_prop_lookup (server, "nautilus:view_as_label_with_mnemonic", langs);
+	viewer_label = bonobo_server_info_prop_lookup (server, "nautilus:viewer_label", langs);
 
         if (view_as_name == NULL) {
-                view_as_name = oaf_server_info_prop_lookup (server, "name", langs);
+                view_as_name = bonobo_server_info_prop_lookup (server, "name", langs);
         }
         if (view_as_name == NULL) {
                 view_as_name = server->iid;
@@ -167,33 +149,37 @@ nautilus_view_identifier_new_from_oaf_server_info (OAF_ServerInfo *server, char
 		}
 		
 		new_identifier = nautilus_view_identifier_new (server->iid, display_name,
-							       view_as_label, viewer_label);
+							       view_as_label,
+							       view_as_label_with_mnemonic,
+							       viewer_label);
 		g_free(display_name);
 		return new_identifier;					
 	}
 		
         return nautilus_view_identifier_new (server->iid, view_as_name,
-					     view_as_label, viewer_label);
+					     view_as_label, 
+					     view_as_label_with_mnemonic,
+					     viewer_label);
 }
 
 NautilusViewIdentifier *
-nautilus_view_identifier_new_from_content_view (OAF_ServerInfo *server)
+nautilus_view_identifier_new_from_content_view (Bonobo_ServerInfo *server)
 {
-	return nautilus_view_identifier_new_from_oaf_server_info
+	return nautilus_view_identifier_new_from_bonobo_server_info
 		(server, "nautilus:view_as_name");
 }
 
 NautilusViewIdentifier *
-nautilus_view_identifier_new_from_property_page (OAF_ServerInfo *server)
+nautilus_view_identifier_new_from_property_page (Bonobo_ServerInfo *server)
 {
-	return nautilus_view_identifier_new_from_oaf_server_info
+	return nautilus_view_identifier_new_from_bonobo_server_info
 		(server, "nautilus:property_page_name");
 }
 
 NautilusViewIdentifier *
-nautilus_view_identifier_new_from_sidebar_panel (OAF_ServerInfo *server)
+nautilus_view_identifier_new_from_sidebar_panel (Bonobo_ServerInfo *server)
 {
-	return nautilus_view_identifier_new_from_oaf_server_info
+	return nautilus_view_identifier_new_from_bonobo_server_info
 		(server, "nautilus:sidebar_panel_name");
 }
 
@@ -204,6 +190,7 @@ nautilus_view_identifier_free (NautilusViewIdentifier *identifier)
                 g_free (identifier->iid);
                 g_free (identifier->name);
                 g_free (identifier->view_as_label);
+		g_free (identifier->view_as_label_with_mnemonic);
                 g_free (identifier->viewer_label);
                 g_free (identifier);
         }
