@@ -34,7 +34,6 @@
 #include "nautilus-application.h"
 #include "nautilus-self-check-functions.h"
 #include "nautilus-window.h"
-#include <X11/Xlib.h>
 #include <bonobo-activation/bonobo-activation.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-ui-main.h>
@@ -93,7 +92,7 @@ eel_gtk_main_quit_all (void)
 	 * This idler will be run by the current event loop, killing it, and then
 	 * by the next event loop, ...
 	 */
-	gtk_idle_add (quit_if_in_main_loop, NULL);
+	g_idle_add (quit_if_in_main_loop, NULL);
 }
 
 static void
@@ -184,7 +183,12 @@ main (int argc, char *argv[])
 				      GNOME_PARAM_POPT_TABLE, options,
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Nautilus"),
 				      NULL);
-
+	
+	/* Need to set this to the canonical DISPLAY value, since
+	   thats where we're registering per-display components */
+	bonobo_activation_set_activation_env_value ("DISPLAY",
+						    gdk_display_get_name (gdk_display_get_default()));
+	
 	g_object_get_property (G_OBJECT (program),
 			       GNOME_PARAM_POPT_CONTEXT,
 			       g_value_init (&context_as_value, G_TYPE_POINTER));
@@ -216,8 +220,6 @@ main (int argc, char *argv[])
 
 	/* Initialize the services that we use. */
 	LIBXML_TEST_VERSION
-	gnome_vfs_init ();
-	bonobo_ui_init ("nautilus", VERSION, &argc, argv);
 
 	if (g_getenv ("NAUTILUS_ENABLE_TEST_COMPONENTS") != NULL) {
 		bonobo_activation_set_test_components_enabled (TRUE);
