@@ -583,7 +583,7 @@ preferences_callback (BonoboUIComponent *component,
 
 	window = GTK_WINDOW (user_data);
 
-	nautilus_file_management_properties_dialog_show (G_CALLBACK (preferences_respond_callback));
+	nautilus_file_management_properties_dialog_show (G_CALLBACK (preferences_respond_callback), window);
 }
 
 static void
@@ -611,6 +611,7 @@ help_menu_about_nautilus_callback (BonoboUIComponent *component,
 		"Andy Hertzfeld",
 		"Arlo Rose",
 		"Darin Adler",
+		"David Camp",
 		"Eli Goldberg",
 		"Elliot Lee",
 		"Eskil Heyn Olsen",
@@ -619,10 +620,13 @@ help_menu_about_nautilus_callback (BonoboUIComponent *component,
 		"George Lebl",
 		"Ian McKellar",
 		"J Shane Culpepper",
+		"James Willcox",
+		"Jan Arne Petersen",
 		"John Harper",
 		"John Sullivan",
 		"Josh Barrow",
 		"Maciej Stachowiak",
+		"Mark McLoughlin",
 		"Mathieu Lacage",
 		"Mike Engber",
 		"Mike Fleming",
@@ -1277,7 +1281,7 @@ void
 nautilus_window_remove_bookmarks_menu_callback (NautilusWindow *window)
 {
         if (window->details->refresh_bookmarks_menu_idle_id != 0) {
-                gtk_idle_remove (window->details->refresh_bookmarks_menu_idle_id);
+                g_source_remove (window->details->refresh_bookmarks_menu_idle_id);
 		window->details->refresh_bookmarks_menu_idle_id = 0;
         }
 }
@@ -1286,7 +1290,7 @@ void
 nautilus_window_remove_go_menu_callback (NautilusWindow *window)
 {
         if (window->details->refresh_go_menu_idle_id != 0) {
-                gtk_idle_remove (window->details->refresh_go_menu_idle_id);
+                g_source_remove (window->details->refresh_go_menu_idle_id);
 		window->details->refresh_go_menu_idle_id = 0;
         }
 }
@@ -1358,8 +1362,8 @@ schedule_refresh_bookmarks_menu (NautilusWindow *window)
 
 	if (window->details->refresh_bookmarks_menu_idle_id == 0) {
                 window->details->refresh_bookmarks_menu_idle_id
-                        = gtk_idle_add (refresh_bookmarks_menu_idle_callback,
-                                        window);
+                        = g_idle_add (refresh_bookmarks_menu_idle_callback,
+				      window);
 	}	
 }
 
@@ -1417,7 +1421,7 @@ schedule_refresh_go_menu (NautilusWindow *window)
 
 	if (window->details->refresh_go_menu_idle_id == 0) {
                 window->details->refresh_go_menu_idle_id
-                        = gtk_idle_add (refresh_go_menu_idle_callback,
-                                        window);
+                        = g_idle_add (refresh_go_menu_idle_callback,
+				      window);
 	}	
 }
